import { EditableCSVViewer } from './widget';
import { DocumentWidget } from '@jupyterlab/docregistry';
import { Signal } from '@lumino/signaling';
import { DATAMODEL_SCHEMA, RECORD_ID } from './model';
/**
 * Responsible for managing the state of the search-and-replace UI element in JupyterLab
 * Contains methods to query a search, highlight the previous/next match,
 * replace the current match, and replace all matches
 */
export class CSVSearchProvider {
    constructor() {
        /**
         * Set to true if the widget under search is read-only, false
         * if it is editable.  Will be used to determine whether to show
         * the replace option.
         */
        this.isReadOnly = false;
        this._matches = [];
        this._changed = new Signal(this);
    }
    /**
     * Report whether or not this provider has the ability to search on the given object
     */
    static canSearchOn(domain) {
        // check to see if the CSVSearchProvider can search on the
        // first cell, false indicates another editor is present
        return (domain instanceof DocumentWidget &&
            domain.content instanceof EditableCSVViewer);
    }
    /**
     * Get an initial query value if applicable so that it can be entered
     * into the search box as an initial query
     *
     * @returns Initial value used to populate the search box.
     */
    getInitialQuery(searchTarget) {
        // CSV Viewer does not support selection
        return null;
    }
    /**
     * Initialize the search using the provided options.  Should update the UI
     * to highlight all matches and "select" whatever the first match should be.
     *
     * @param query A RegExp to be use to perform the search
     * @param searchTarget The widget to be searched
     *
     * @returns A promise that resolves with a list of all matches
     */
    async startQuery(query, searchTarget) {
        this._target = searchTarget;
        this._query = query;
        // when changes are made to the datamodel, rerun the search
        searchTarget.content.dataModel.changed.connect(this.rerunSearch, this);
        // query for the matches in the model data
        searchTarget.content.searchService.find(query);
        // update match variables in CSVSearchProvider
        this._matches = searchTarget.content.searchService.matches;
        this._currentMatch = searchTarget.content.searchService.currentMatch;
        this.selectSingleCell();
        return this._matches;
    }
    /**
     * Clears state of a search provider to prepare for startQuery to be called
     * in order to start a new query or refresh an existing one.
     *
     * @returns A promise that resolves when the search provider is ready to
     * begin a new search.
     */
    async endQuery() {
        this._matches = [];
        this._currentMatch = null;
        this._target.content.dataModel.changed.disconnect(this.rerunSearch, this);
        this._target.content.searchService.clear();
    }
    /**
     * Resets UI state as it was before the search process began.  Cleans up and
     * disposes of all internal state.
     *
     * @returns A promise that resolves when all state has been cleaned up.
     */
    async endSearch() {
        this._target.content.searchService.clear();
        this._target.content.dataModel.changed.disconnect(this.rerunSearch, this);
        this._query = undefined;
    }
    /**
     * Move the current match indicator to the next match.
     *
     * @returns A promise that resolves once the action has completed.
     */
    async highlightNext() {
        this._currentMatch = this._target.content.searchService.highlightNext(false);
        this.selectSingleCell();
        return this._currentMatch;
    }
    /**
     * Move the current match indicator to the previous match.
     *
     * @returns A promise that resolves once the action has completed.
     */
    async highlightPrevious() {
        this._currentMatch = this._target.content.searchService.highlightNext(true);
        this.selectSingleCell();
        return this._currentMatch;
    }
    /**
     * Replace the currently selected match with the provided text
     *
     * @returns A promise that resolves once the action has completed.
     */
    async replaceCurrentMatch(newText, useLitestore = true) {
        const { line, column } = this._target.content.searchService.currentMatch;
        await this._target.content.dataModel.setData('body', line, column, newText, useLitestore);
        this.selectSingleCell();
        return true;
    }
    /**
     * Replace all matches in the notebook with the provided text
     * Not implemented in the CSV viewer as it is read-only.
     *
     * @returns A promise that resolves once the action has completed.
     */
    async replaceAllMatches(newText) {
        const model = this._target.content.dataModel.dsvModel;
        const litestore = this._target.content.dataModel.litestore;
        const searchService = this._target.content.searchService;
        const startRow = searchService.currentMatch.line;
        const startColumn = searchService.currentMatch.column;
        let endRow, endColumn;
        litestore.beginTransaction();
        // replace every match individually
        while (this.matches.length > 0) {
            // when we have one match left, get the last row and column being edited
            if (this.matches.length === 1) {
                endRow = searchService.currentMatch.line;
                endColumn = searchService.currentMatch.column;
            }
            this.replaceCurrentMatch(newText, false);
        }
        litestore.updateRecord({
            schema: DATAMODEL_SCHEMA,
            record: RECORD_ID
        }, {
            modelData: model.rawData,
            modelHeader: model.header,
            change: {
                type: 'cells-changed',
                region: 'body',
                // the range of cells being edited
                row: startRow,
                column: startColumn,
                rowSpan: endRow - startRow,
                columnSpan: endColumn - startColumn
            }
        });
        litestore.endTransaction();
        return true;
    }
    /**
     * Reruns the search query when changes a made
     * Used when changes are made to the data model
     */
    rerunSearch() {
        this.endQuery();
        this.startQuery(this._query, this._target);
        this._changed.emit(undefined);
        this.selectSingleCell();
        return true;
    }
    /**
     * Selects the cell that is the current match
     */
    selectSingleCell() {
        if (this._target) {
            const { line, column } = this._currentMatch;
            this._target.content.selectSingleCell(line, column);
        }
    }
    /**
     * Signal indicating that something in the search has changed, so the UI should update
     */
    get changed() {
        return this._changed;
    }
    /**
     * The same list of matches provided by the startQuery promise resolution
     */
    get matches() {
        // Ensure that no other fn can overwrite matches index property
        // We shallow clone each node
        return this._matches
            ? this._matches.map(m => Object.assign({}, m))
            : this._matches;
    }
    /**
     * The current index of the selected match.
     */
    get currentMatchIndex() {
        if (!this._currentMatch) {
            return null;
        }
        return this._currentMatch.index;
    }
    get currentMatch() {
        return this._currentMatch;
    }
}
//# sourceMappingURL=searchprovider.js.map